pTune           EQU     $00
pChanCount      EQU     $02
pChanNote       EQU     $03
pChanVol        EQU     $04
pChanNumber     EQU     $05

pTuneVolume     EQU     $00
pNoiseNote      EQU     $01
pNoteOffset     EQU     $02
pNotePtr        EQU     $03

SoundPort       EQU     $FE
Speed           EQU     $FE
BorderColour    EQU     $00

                segment code

Note            MACRO(Freq,Length)
                DEFB    Freq*$08+Length
                ENDM

AddTune:        ADD     A,A
                LD      L,A
                ADD     A,A
                ADD     A,L
                ADD     A,LOW BufferOne
                LD      IXL,A
                LD      A,HIGH BufferOne
                ADC     A,$00
                LD      IXH,A
                LD      L,(IX+pTune+$00)
                LD      H,(IX+pTune+$01)
                OR      A
                SBC     HL,DE
                RET     Z
                DI
                LD      (IX+pTune+$00),E
                LD      (IX+pTune+$01),D
                LD      (IX+pChanCount),$00
                LD      HL,pNotePtr
                ADD     HL,DE
                LD      (HL),pNotePtr
                EI
                RET

KillSound:      LD      HL,DeadTune
                LD      (BufferOne),HL
                LD      (BufferTwo),HL
                RET

DoSound:        EX      AF,AF'
                PUSH    AF
                PUSH    IX
                PUSH    IY
                LD      IX,BufferTwo
                LD      L,(IX+pTune+$00)
                LD      H,(IX+pTune+$01)
                PUSH    HL
                POP     IY
                LD      DE,DeadTune
                OR      A
                SBC     HL,DE
                JR      Z,DoTheTune
                CALL    PlayNextNote
                LD      A,$00
                OUT     (SoundPort),A
                POP     IY
                POP     IX
                POP     AF
                EX      AF,AF'
                RET

DoTheTune:      LD      IX,BufferOne
                LD      L,(IX+pTune+$00)
                LD      H,(IX+pTune+$01)
                PUSH    HL
                LD      DE,DeadTune
                OR      A
                SBC     HL,DE
                POP     IY
                CALL    NZ,PlayNextNote
                LD      A,$00
                OUT     (SoundPort),A
                POP     IY
                POP     IX
                POP     AF
                EX      AF,AF'
                RET

DoChannel:      LD      A,(IX+pChanCount)
                OR      A
                JR      Z,GetNextNote
                DEC     (IX+pChanCount)
                LD      A,(IX+pChanNote)
                OR      A
                RET     NZ
                LD      A,(IX+pChanVol)
                OR      A
                LD      A,(IX+pChanNote)
                RET     Z
                DEC     (IX+pChanVol)
                RET

GetNextNote:    INC     (IY+pNotePtr)
                LD      A,(IY+pNotePtr)
                ADD     A,IYL
                LD      L,A
                LD      A,IYH
                ADC     A,$00
                LD      H,A
                LD      A,(HL)
                CP      $FF
                JR      Z,EndTune
                CP      $FE
                JR      NZ,NewNote
                LD      (IY+pNotePtr),pNotePtr
                JP      GetNextNote

EndTune:        LD      HL,DeadTune
                LD      (IX+pTune+$00),L
                LD      (IX+pTune+$01),H
                LD      A,pNotePtr
                LD      (DeadTune+pNotePtr),A
                RET

NewNote:        LD      C,A
                AND     $07
                ADD     A,LOW LenTable
                LD      L,A
                ADC     A,HIGH LenTable
                SUB     L
                LD      H,A
                LD      A,(HL)
                LD      (IX+pChanCount),A
                LD      A,C
                RRA
                RRA
                RRA
                AND     $1F
                LD      (IX+pChanNote),A
                RET

GetFrequency:   ADD     A,$0C
                LD      E,A
                LD      D,$00
                LD      HL,FreqTable
                ADD     HL,DE
                LD      H,(HL)
                LD      L,$01
                RET

PlayNextNote:   CALL    DoChannel
                ADD     A,(IY+pNoteOffset)
                CALL    GetFrequency
                EX      DE,HL
                LD      B,$00
                LD      A,BorderColour
                LD      H,D
                LD      D,$10
OuterSound:     DEC     E
                OUT     (SoundPort),A   ;Output on channel one
                JR      NZ,NoFlipOne
                LD      E,H
                XOR     D
NoFlipOne:      NOP
                NOP
                NOP
                NOP
                DJNZ    OuterSound
                RET


FreqTable:      DEFB    $FF,$F0,$E3,$D7,$CB,$C0,$B4,$AB
                DEFB    $A1,$97,$90,$88,$80,$79,$72,$6C
                DEFB    $66,$60,$5B,$56,$51,$4C,$48,$44
                DEFB    $40,$3D,$39,$36,$33,$30,$2D,$2B
                DEFB    $28,$26,$24,$22,$20,$1E,$1C,$1B
                DEFB    $19,$18,$17,$15,$14,$13,$12,$11
                DEFB    $10,$0F,$0E,$0D,$0C,$01

BufferOne:      DEFW    DeadTune
                DEFB    $00,$00,$00,$00
BufferTwo:      DEFW    DeadTune
                DEFB    $00,$00,$00,$01

DeadTune:       DEFB    $00,$00,$00,$03
                DEFB    $00,$FF

LenTable:       DEFB    $03,$07,$0B,$0F,$13,$17,$1B,$1F

HitTune:        DEFB    $0C,$09,$0C,$07
                Note($0C,$00)
                DEFB    $FF

KillItTune:     DEFB    $0C,$01,$18,$07
                Note(   $02,$02)
                Note(   $04,$02)
                Note(   $06,$02)
                DEFB    $FF

PickTune:       DEFB    $0C,$01,$18,$07
                Note($0C,$01)
                DEFB    $FF

DropTune:       DEFB    $0C,$01,$0C,$07
                Note($0C,$01)
                DEFB    $FF

Nos1Tune:       DEFB    $08,$01,$0C,$03
                Note(   $0C,$03)
                Note(   $0F,$03)
                Note(   $13,$03)
                Note(   $14,$03)

                Note(   $0B,$03)
                Note(   $13,$03)
                Note(   $12,$03)
                Note(   $11,$03)

                Note(   $10,$03)
                Note(   $0F,$03)
                Note(   $0F,$01)
                Note(   $0E,$01)
                Note(   $0D,$01)
                Note(   $0C,$01)

                Note(   $0B,$01)
                Note(   $09,$00)
                Note(   $07,$00)
                Note(   $0C,$01)
                Note(   $11,$01)
                Note(   $10,$01)
                Note(   $0F,$01)
                Note(   $0E,$03)

                Note(   $0C,$03)
                Note(   $0C,$00)
                Note(   $0B,$00)
                Note(   $0C,$00)
                Note(   $0E,$00)
                Note(   $0F,$05)
                Note(   $11,$01)

                Note(   $13,$03)
                Note(   $13,$00)
                Note(   $0F,$00)
                Note(   $11,$00)
                Note(   $13,$00)
                Note(   $14,$05)
                Note(   $13,$01)

                Note(   $11,$01)
                Note(   $14,$01)
                Note(   $17,$01)
                Note(   $1A,$01)
                Note(   $0F,$01)
                Note(   $13,$01)
                Note(   $17,$01)
                Note(   $18,$01)

                Note(   $0E,$01)
                Note(   $18,$03)
                Note(   $17,$01)
                Note(   $18,$01)
                Note(   $13,$01)
                Note(   $0F,$01)
                Note(   $13,$01)

                DEFB    $FE

